unit Config;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Spin, IniFiles, Main, ExtCtrls, ComCtrls, ShellAPI, Global;

type
  TConfigForm = class(TForm)
    OkBtn: TButton;
    CancelBtn: TButton;
    DefaultBtn: TButton;
    Label1: TLabel;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    CharacterDensityLbl: TLabel;
    FallingSpeedLbl: TLabel;
    FallingCodeDurationLbl: TLabel;
    WritingSpeedLbl: TLabel;
    ShowFPSCheckBox: TCheckBox;
    CharacterDensityTrackBar: TTrackBar;
    FallingSpeedTrackBar: TTrackBar;
    FallingCodeDurationTrackBar: TTrackBar;
    WritingSpeedTrackBar: TTrackBar;
    Bevel1: TBevel;
    PhoneNumberLbl: TLabel;
    FindingNumberSpeedLbl: TLabel;
    FindingNumberSpeedTrackBar: TTrackBar;
    PhoneNumberEdt: TEdit;
    ActivatePhoneSequenceCheckBox: TCheckBox;
    Bevel2: TBevel;
    ActivateTrinityTextCheckBox: TCheckBox;
    ChangingWindowFrequencyLbl: TLabel;
    ChangingWindowFrequencyTrackBar: TTrackBar;
    Label2: TLabel;
    FallingTextEdt: TEdit;
    CharacterDensityDeltaTrackBar: TTrackBar;
    FallingSpeedDeltaTrackBar: TTrackBar;
    FallingCodeDurationDeltaTrackBar: TTrackBar;
    TrinityTextLbl: TLabel;
    TrinityTextMemo: TMemo;
    Label3: TLabel;
    ResolCb: TComboBox;
    procedure LoadConfig;
    procedure SaveConfig;
    procedure FormCreate(Sender: TObject);
    procedure BtnClick(Sender: TObject);
    procedure PhoneNumberEdtKeyPress(Sender: TObject; var Key: Char);
    procedure CheckBoxClick(Sender: TObject);
    procedure Label1Click(Sender: TObject);
    procedure TrackBarChange(Sender: TObject);
    procedure ResolCbChange(Sender: TObject);
  private
    { Dclarations prives}
  public
    { Dclarations publiques}
  end;

const
  crHand = 1;

var
  ConfigForm: TConfigForm;

implementation

{$R *.DFM}

//-----------------------------------------------------------------------------

procedure TConfigForm.LoadConfig;
var
  IniFile: TIniFile;
  I: Integer;
begin
  IniFile := TIniFile.Create(CfgFile);
  with IniFile do
  try
    ResolCb.ItemIndex := ReadInteger('Common', 'Resolution', RESOL_DEF);
    ShowFPSCheckBox.Checked := ReadBool('Common', 'ShowFPS', SHOW_FPS_DEF);
    CharacterDensityTrackBar.Position := ReadInteger('Code', 'CharacterDensity', CHARACTER_DENSITY_DEF);
    CharacterDensityDeltaTrackBar.Position := ReadInteger('Code', 'CharacterDensityDelta', CHARACTER_DENSITY_DELTA_DEF);
    FallingSpeedTrackBar.Position := ReadInteger('Code', 'FallingSpeed', FALLING_SPEED_DEF);
    FallingSpeedDeltaTrackBar.Position := ReadInteger('Code', 'FallingSpeedDelta', FALLING_SPEED_DELTA_DEF);
    FallingCodeDurationTrackBar.Position := ReadInteger('Code', 'FallingCodeDuration', FALLING_CODE_DURATION_DEF);
    FallingCodeDurationDeltaTrackBar.Position := ReadInteger('Code', 'FallingCodeDurationDelta', FALLING_CODE_DURATION_DELTA_DEF);
    ChangingWindowFrequencyTrackBar.Position := ReadInteger('Code', 'ChangingWindowFrequency', CHANGING_WINDOW_FREQUENCY_DEF);
    FallingTextEdt.Text := ReadString('Code', 'FallingText', FALLING_TEXT_DEF);
    ActivatePhoneSequenceCheckBox.Checked := ReadBool('Phone', 'PhoneSequenceEnabled', PHONE_SEQUENCE_ENABLED_DEF);
    PhoneNumberEdt.Text := ReadString('Phone', 'PhoneNumber', PHONE_NUMBER_DEF);
    FindingNumberSpeedTrackBar.Position := ReadInteger('Phone', 'FindingNumberSpeed', FINDING_NUMBER_SPEED_DEF);
    ActivateTrinityTextCheckBox.Checked := ReadBool('Trinity', 'TrinityTextEnabled', TRINITY_TEXT_ENABLED_DEF);
    WritingSpeedTrackBar.Position := ReadInteger('Trinity', 'WritingSpeed', WRITING_SPEED_DEF);
    I := 0;
    while ValueExists('Trinity', 'Line' + IntTostr(I)) do
    begin
      TrinityTextMemo.Lines.Add(ReadString('Trinity', 'Line' + IntToStr(I), ''));
      Inc(I);
    end;
  finally
    IniFile.Free;
  end;
  CheckBoxClick(ActivatePhoneSequenceCheckBox);
  CheckBoxClick(ActivateTrinityTextCheckBox);
end;

//-----------------------------------------------------------------------------

procedure TConfigForm.SaveConfig;
var
  IniFile: TIniFile;
  EndText: string;
  I: Integer;
begin
  case ResolCb.ItemIndex of
    0: MATRIX_WIDTH := 800 div GLYPH_WIDTH; //800x600
    2: MATRIX_WIDTH := 1152 div GLYPH_WIDTH; //1152x864
    3: MATRIX_WIDTH := 1280 div GLYPH_WIDTH; //1280x1024
    4: MATRIX_WIDTH := 1600 div GLYPH_WIDTH; //1600x1200
    else MATRIX_WIDTH := 1024 div GLYPH_WIDTH; //1024x768
  end;
  IniFile := TIniFile.Create(CfgFile);
  try
    with IniFile do begin
      WriteInteger('Common', 'Resolution', ResolCb.ItemIndex);
      WriteBool('Common', 'ShowFPS', ShowFPSCheckBox.Checked);
      WriteInteger('Code', 'CharacterDensity', CharacterDensityTrackBar.Position);
      WriteInteger('Code', 'CharacterDensityDelta', CharacterDensityDeltaTrackBar.Position);
      WriteInteger('Code', 'FallingSpeed', FallingSpeedTrackBar.Position);
      WriteInteger('Code', 'FallingSpeedDelta', FallingSpeedDeltaTrackBar.Position);
      WriteInteger('Code', 'FallingCodeDuration', FallingCodeDurationTrackBar.Position);
      WriteInteger('Code', 'FallingCodeDurationDelta', FallingCodeDurationDeltaTrackBar.Position);
      WriteInteger('Code', 'ChangingWindowFrequency', ChangingWindowFrequencyTrackBar.Position);
      EndText := FallingTextEdt.Text;
      while EndText[1] = '_' do Delete(EndText, 1, 1);
      while EndText[Length(EndText)] = '_' do Delete(EndText, Length(EndText), 1);
      for I := 1 to Length(EndText) do
        if not (EndText[I] in ['!'..'~']) then
          EndText[I] := '_';
      if Length(EndText) > MATRIX_WIDTH then
        EndText := Copy(EndText, 1, MATRIX_WIDTH)
      else if Length(EndText) < MATRIX_WIDTH then
      begin
        for I := 1 to (MATRIX_WIDTH - Length(EndText)) div 2 do
          EndText := '_' + EndText + '_';
        while Length(EndText) < MATRIX_WIDTH do
          EndText := '_' + EndText;
      end;
      WriteString('Code', 'FallingText', EndText);
      WriteBool('Phone', 'PhoneSequenceEnabled', ActivatePhoneSequenceCheckBox.Checked);
      WriteString('Phone', 'PhoneNumber', PhonenUmberEdt.Text);
      WriteInteger('Phone', 'FindingNumberSpeed', FindingNumberSpeedTrackBar.Position);
      WriteBool('Trinity', 'TrinityTextEnabled', ActivateTrinityTextCheckBox.Checked);
      WriteInteger('Trinity', 'WritingSpeed', WritingSpeedTrackBar.Position);
      while TrinityTextMemo.Lines.IndexOf('') > -1 do
        TrinityTextMemo.Lines.Delete(TrinityTextMemo.Lines.IndexOf(''));
      if TrinityTextMemo.Lines.Count < 1 then
        TrinityTextMemo.Lines.Add(TRINITY_TEXT_DEF);
      for I := 0 to TrinityTextMemo.Lines.Count - 1 do
        WriteString('Trinity', 'Line' + IntToStr(I), TrinityTextMemo.Lines[I]);
      I := TrinityTextMemo.Lines.Count;
      while ValueExists('Trinity', 'Line' + IntToStr(I)) do
      begin
        DeleteKey('Trinity', 'Line' + IntToStr(I));
        Inc(I);
      end;
    end;
  finally
    IniFile.Free;
  end;
end;

//-----------------------------------------------------------------------------

procedure TConfigForm.FormCreate(Sender: TObject);
begin
  Screen.Cursors[crHand] := LoadCursor(0, IDC_HAND);
  Label1.Cursor := crHand;
  ClientHeight := OkBtn.Top + OkBtn.Height + 8;
  ClientWidth := DefaultBtn.Left + DefaultBtn.Width + 8;
  PageControl1.ActivePageIndex := 0;
  LoadConfig;
  TrackBarChange(nil);
end;

//-----------------------------------------------------------------------------

procedure TConfigForm.BtnClick(Sender: TObject);
begin
  if (Sender = OkBtn) then
  begin
    SaveConfig;
    Close;
  end
  else if (Sender = CancelBtn) then
  begin
    Close;
  end
  else if (Sender = DefaultBtn) then
  begin
    CharacterDensityTrackBar.Position := CHARACTER_DENSITY_DEF;
    CharacterDensityDeltaTrackBar.Position := CHARACTER_DENSITY_DELTA_DEF;
    FallingSpeedTrackBar.Position := FALLING_SPEED_DEF;
    FallingSpeedDeltaTrackBar.Position := FALLING_SPEED_DELTA_DEF;
    FallingCodeDurationTrackBar.Position := FALLING_CODE_DURATION_DEF;
    FallingCodeDurationDeltaTrackBar.Position := FALLING_CODE_DURATION_DELTA_DEF;
    ChangingWindowFrequencyTrackBar.Position := CHANGING_WINDOW_FREQUENCY_DEF;
    FallingTextEdt.Text := FALLING_TEXT_DEF;
    PhoneNumberEdt.Text := PHONE_NUMBER_DEF;
    FindingNumberSpeedTrackBar.Position := FINDING_NUMBER_SPEED_DEF;
    WritingSpeedTrackBar.Position := WRITING_SPEED_DEF;
    ShowFPSCheckBox.Checked := SHOW_FPS_DEF;
    ActivatePhoneSequenceCheckBox.Checked := PHONE_SEQUENCE_ENABLED_DEF;
    ActivateTrinityTextCheckBox.Checked := TRINITY_TEXT_ENABLED_DEF;
    TrinityTextMemo.Lines.SetText(TRINITY_TEXT_DEF);
  end;
  CheckBoxClick(ActivatePhoneSequenceCheckBox);
  CheckBoxClick(ActivateTrinityTextCheckBox);
end;

//-----------------------------------------------------------------------------

procedure TConfigForm.PhoneNumberEdtKeyPress(Sender: TObject;
  var Key: Char);
begin
  if not (Key in ['0'..'9', #8, #9, #13]) then Key := Chr(0);
end;

//-----------------------------------------------------------------------------

procedure TConfigForm.CheckBoxClick(Sender: TObject);
begin
  if (Sender = ActivatePhoneSequenceCheckBox) then
  begin
    PhoneNumberLbl.Enabled := ActivatePhoneSequenceCheckBox.Checked;
    PhoneNumberEdt.Enabled := ActivatePhoneSequenceCheckBox.Checked;
    FindingNumberSpeedLbl.Enabled := ActivatePhoneSequenceCheckBox.Checked;
    FindingNumberSpeedTrackBar.Enabled := ActivatePhoneSequenceCheckBox.Checked;
  end
  else if (Sender = ActivateTrinityTextCheckBox) then
  begin
    WritingSpeedLbl.Enabled := ActivateTrinityTextCheckBox.Checked;
    WritingSpeedTrackBar.Enabled := ActivateTrinityTextCheckBox.Checked;
    TrinityTextLbl.Enabled := ActivateTrinityTextCheckBox.Checked;
    TrinityTextMemo.Enabled := ActivateTrinityTextCheckBox.Checked;
  end;
end;

procedure TConfigForm.Label1Click(Sender: TObject);
begin
  ShellExecute(0, 'open', PChar(Label1.Caption), nil, nil, SW_SHOWNORMAL);
end;

procedure TConfigForm.TrackBarChange(Sender: TObject);
begin
  CharacterDensityTrackBar.SelStart := CharacterDensityTrackBar.Position -
    CharacterDensityTrackBar.LineSize * CharacterDensityDeltaTrackBar.Position div 2;
  CharacterDensityTrackBar.SelEnd := CharacterDensityTrackBar.Position +
    CharacterDensityTrackBar.LineSize * CharacterDensityDeltaTrackBar.Position div 2;
  FallingSpeedTrackBar.SelStart := FallingSpeedTrackBar.Position -
    FallingSpeedTrackBar.LineSize * FallingSpeedDeltaTrackBar.Position div 2;
  FallingSpeedTrackBar.SelEnd := FallingSpeedTrackBar.Position +
    FallingSpeedTrackBar.LineSize * FallingSpeedDeltaTrackBar.Position div 2;
  FallingCodeDurationTrackBar.SelStart := FallingCodeDurationTrackBar.Position -
    FallingCodeDurationTrackBar.LineSize * FallingCodeDurationDeltaTrackBar.Position div 2;
  FallingCodeDurationTrackBar.SelEnd := FallingCodeDurationTrackBar.Position +
    FallingCodeDurationTrackBar.LineSize * FallingCodeDurationDeltaTrackBar.Position div 2;
end;

procedure TConfigForm.ResolCbChange(Sender: TObject);
var
  I: Integer;
  EndText: string;
begin
  case ResolCb.ItemIndex of
    0: MATRIX_WIDTH := 800 div GLYPH_WIDTH; //800x600
    2: MATRIX_WIDTH := 1152 div GLYPH_WIDTH; //1152x864
    3: MATRIX_WIDTH := 1280 div GLYPH_WIDTH; //1280x1024
    4: MATRIX_WIDTH := 1600 div GLYPH_WIDTH; //1600x1200
    else MATRIX_WIDTH := 1024 div GLYPH_WIDTH; //1024x768
  end;
  EndText := FallingTextEdt.Text;
  while EndText[1] = '_' do Delete(EndText, 1, 1);
  while EndText[Length(EndText)] = '_' do Delete(EndText, Length(EndText), 1);
  for I := 1 to Length(EndText) do
    if not (EndText[I] in ['!'..'~']) then
      EndText[I] := '_';
  if Length(EndText) > MATRIX_WIDTH then
    EndText := Copy(EndText, 1, MATRIX_WIDTH)
  else if Length(EndText) < MATRIX_WIDTH then
  begin
    for I := 1 to (MATRIX_WIDTH - Length(EndText)) div 2 do
      EndText := '_' + EndText + '_';
    while Length(EndText) < MATRIX_WIDTH do
      EndText := '_' + EndText;
  end;
  FallingTextEdt.Text := EndText;
end;

end.

