unit Code;

interface

uses Classes, Windows, DXClass, Global, Sysutils;

type
  //un caractere : la matrice est un tableau de caracteres
  TChar = record
    IsLatin: Boolean;
    CharNum: Integer;
    Changing: Integer;
    HighLighted: Integer;
    Visible: Integer;
    BlackTemp: Integer;
    BlackTempVisible: Integer;
  end;

  //etat dans lequel se trouve la matrice
  TMatrixState = (msFalling, msFallEnding, msFallEnded, msOpeningNewWindow,
    msOpeningNewWindow2, msStopped, msDejaVu);

  //type ancetre de la matrice
  TRows = class
    TheMatrix: array[0..MATRIX_WIDTH - 1, 0..MATRIX_HEIGHT - 1] of TChar;
    function RowFree(X: ShortInt; Speed: Integer): Boolean; virtual;
    function ShortRowFree(X, Y1, Y2: ShortInt): Boolean; virtual;
  end;

  //une colonne : chaque suite de caracteres qui tombent est une colonne
  TRow = class
  private
    FParent: TRows;
    FChar: Char;
    FY1, FY2: Integer;
    FHighlight: Boolean;
  public
    X, Y: Integer;
    Speed: Integer;
    Pos: Integer;
    Height: Integer;
    constructor Create(Parent: TRows);
    function Init(NbRow: Integer = -1; Letter: Char = ' '): Boolean;
    procedure StepIt(ReInit: Boolean = True);
  end;

  TShortRow = class
  private
    FParent: TRows;
    FBlackSection: Integer;
  public
    X, Y1, Y2: Integer;
    Speed: Integer;
    Pos: Integer;
    constructor Create(Parent: TRows);
    procedure Init;
    procedure StepIt(ReInit: Boolean = True);
  end;

  //la matrice
  TCodeEmulator = class(TRows)
  private
    FMatrixState: TMatrixState;
    FRows: array[0..NB_ROW_MAX - 1] of TRow;
    FShortRows: array[0..NB_SHORT_ROW - 1] of TShortRow;
    FNbRow: Integer;
    FInterval: Integer;
    FNbIterationsMax: Integer;
    FNbIterations: Integer;
    FNbIter: Integer;
    FChangingWindowFrequency: Integer;
    FChangingWindowFrequency2: Integer;
    FFallingEndText: string;
    FErasingEndGlyph: Boolean;
    FTimer: TDXTimer;
  public
    EndText: string[MATRIX_WIDTH];
    EndTextInit: array[1..MATRIX_WIDTH] of Boolean;
    constructor Create(Timer: TDXTimer);
    destructor Destroy; override;
    function RowFree(X: ShortInt; Speed: Integer): Boolean; override;
    function ShortRowFree(X, Y1, Y2: ShortInt): Boolean; override;
    procedure NewSequence;
    procedure StepIt(Sender: TObject; LagCount: Integer);
    procedure OpenWindow;
  end;

implementation

uses Main, Math;

const
  MatrixStateStr: array[TMatrixState] of string = ('msFalling', 'msFallEnding',
    'msFallEnded', 'msOpeningNewWindow', 'msOpeningNewWindow2', 'msStopped',
    'msDejaVu');

  //-----------------------------------------------------------------------------

function TRows.RowFree(X: ShortInt; Speed: Integer): Boolean;
begin
  Result := False;
end;

//-----------------------------------------------------------------------------

function TRows.ShortRowFree(X, Y1, Y2: ShortInt): Boolean;
begin
  Result := False;
end;

//-----------------------------------------------------------------------------

constructor TRow.Create(Parent: TRows);
begin
  FParent := Parent;
end;

//-----------------------------------------------------------------------------

function TRow.Init(NbRow: Integer = -1; Letter: Char = ' '): Boolean;
var
  NbTry: Integer;
begin
  Height := Random(3 * MATRIX_HEIGHT) + 15;
  if (NbRow = -1) and (Random(5) = 0) then
    case Random(3) of
      0:
        begin
          FY1 := Random(MATRIX_HEIGHT div 2);
          FY2 := MATRIX_HEIGHT;
        end;
      1:
        begin
          FY1 := -1;
          FY2 := MATRIX_HEIGHT div 2 + Random(MATRIX_HEIGHT div 2);
        end;
      else
        begin
          FY1 := Random(MATRIX_HEIGHT div 2 - 5);
          FY2 := MATRIX_HEIGHT div 2 + Random(MATRIX_HEIGHT div 2 - 5);
        end;
    end
  else
  begin
    FY1 := -1;
    FY2 := MATRIX_HEIGHT;
  end;
  Speed := Random(SPEED_MAX);
  FHighlight := Random(5) = 0;
  Pos := Random(Speed);
  Result := True;
  FChar := ' ';
  if (NbRow <> -1) then
    if FParent.RowFree(NbRow, Speed) then
    begin
      Height := Random(MATRIX_HEIGHT div 2 - 5) + 10;
      X := NbRow;
      Y := -1;
      FChar := Letter;
    end
    else
      Result := False
  else
  begin
    NbTry := 1;
    repeat
      X := Random(MATRIX_WIDTH);
      Inc(NbTry);
    until ((NbTry > 40) or FParent.RowFree(X, Speed));
    if (NbTry <= 40) then
      Y := -1
    else
      Result := False;
  end;
end;

//-----------------------------------------------------------------------------

procedure TRow.StepIt(ReInit: Boolean = True);
var
  Decal: Integer;
begin
  Inc(Pos);
  if Y in [0..MATRIX_HEIGHT - 1] then
    FParent.TheMatrix[X, Y].CharNum := Random(NB_GLYPH);
  if (Pos > Speed) then
  begin
    if (FChar = ' ') or (Y < MATRIX_HEIGHT div 2) then
      Inc(Y);
    Pos := 0;
    if (Y < FY2) then
      if (FChar <> ' ') then
      begin
        if (Y > FY1) then
        begin
          FParent.TheMatrix[X, Y - 1].IsLatin := False;
          FParent.TheMatrix[X, Y - 1].CharNum := Random(NB_GLYPH);
          FParent.TheMatrix[X, Y - 1].Changing := SPEED_MAX;
        end;
        FParent.TheMatrix[X, Y].IsLatin := True;
        FParent.TheMatrix[X, Y].CharNum := Ord(FChar) - Ord(' ');
        FParent.TheMatrix[X, Y].Changing := 0;
        FParent.TheMatrix[X, Y].Visible := IfThen(FHighlight, -1, 1) * Speed *
          Height + Random(10);
        if Y > MATRIX_HEIGHT div 2 - 2 div 2 then
          Y := MATRIX_HEIGHT + 1;
      end
      else
      begin
        if (Random(25) = 0) and (X > 0) then
          Decal := -1
        else if (Random(25) = 0) and (X < MATRIX_WIDTH - 1) then
          Decal := 1
        else
          Decal := 0;
        if (Y > FY1) and (Random(15) > 0) then
        begin
          FParent.TheMatrix[X + Decal, Y].Visible := IfThen(FHighlight, -1, 1) *
            Speed * Height + Random(10);
          FParent.TheMatrix[X + Decal, Y].HighLighted := Speed * 2;
        end;
      end
    else if ReInit then
      Init;
  end;
end;

//-----------------------------------------------------------------------------

constructor TShortRow.Create(Parent: TRows);
begin
  FParent := Parent;
  X := -1;
  Y1 := -1;
  Y2 := -1;
end;

//-----------------------------------------------------------------------------

procedure TShortRow.Init;
var
  NbTry: Integer;
begin
  NbTry := 0;
  repeat
    Inc(NbTry);
    X := Random(MATRIX_WIDTH);
    Y1 := Random(MATRIX_HEIGHT - 15);
    Y2 := Y1 + 5 + Random(10);
  until (NbTry > 40) or ((Y2 < MATRIX_HEIGHT) and (FParent.ShortRowFree(X, Y1, Y2)));
  if (NbTry <= 40) then
  begin
    Speed := SPEED_MAX;
    Pos := Random(Speed);
    FBlackSection := 0;
  end
  else
  begin
    X := -1;
    Y1 := -1;
    Y2 := -1;
  end;
end;

//-----------------------------------------------------------------------------

procedure TShortRow.StepIt(ReInit: Boolean = True);
var
  I: Integer;
begin
  if X < 0 then Exit;
  Inc(Pos);
  if (Pos > Speed) then
  begin
    Pos := 0;
    for I := Y2 downto Y1 + 1 do
    begin
      FParent.TheMatrix[X, I].CharNum := FParent.TheMatrix[X, I - 1].CharNum;
      FParent.TheMatrix[X, I].Changing := SPEED_MAX;
      if FParent.TheMatrix[X, I - 1].BlackTemp > 0 then
      begin
        FParent.TheMatrix[X, I].BlackTemp := FParent.TheMatrix[X, I - 1].BlackTemp + Speed;
        FParent.TheMatrix[X, I].BlackTempVisible := FParent.TheMatrix[X, I - 1].BlackTempVisible +
          Speed;
      end;
    end;
    if FBlackSection > 0 then
      Dec(FBlackSection);
    FParent.TheMatrix[X, Y1].CharNum := Random(NB_GLYPH);
    if Random(6) = 0 then
    begin
      FBlackSection := Random(5);
      FParent.TheMatrix[X, Y1].BlackTemp := FBlackSection * SPEED_MAX;
      FParent.TheMatrix[X, Y1].BlackTempVisible := 2 * SPEED_MAX;
    end;
    if ReInit then
      Init;
  end;
end;

//-----------------------------------------------------------------------------

constructor TCodeEmulator.Create(Timer: TDXTimer);
var
  I: Integer;
begin
  FFallingEndText := FallingEndText;
  FTimer := Timer;
  for I := 0 to NB_ROW_MAX do
    FRows[I] := TRow.Create(Self);
  for I := 0 to NB_SHORT_ROW - 1 do
    FShortRows[I] := TShortRow.Create(Self);
end;

//-----------------------------------------------------------------------------

function TCodeEmulator.RowFree(X: ShortInt; Speed: Integer): Boolean;
var
  I: Integer;
begin
  Result := True;
  for I := 0 to FNbRow - 1 do
    if ((FRows[I].X = X) and ((FRows[I].Y < 5) or ((MATRIX_HEIGHT - FRows[I].Y) *
      FRows[I].Speed >= MATRIX_HEIGHT * Speed))) then
      Result := False;
end;

//-----------------------------------------------------------------------------

function TCodeEmulator.ShortRowFree(X, Y1, Y2: ShortInt): Boolean;
begin
  Result := True;
end;

//-----------------------------------------------------------------------------

destructor TCodeEmulator.Destroy;
begin
  inherited Destroy;
end;

//-----------------------------------------------------------------------------

procedure TCodeEmulator.NewSequence;
var
  I, J: Integer;
begin
  FNbRow := CHARACTER_DENSITY_DEF;
  FNbRow := NbRowMini + Random(NbRowRand);
  FInterval := 60 - (CodeIntervalMini + Random(CodeIntervalRand));
  FNbIterationsMax := (NbIterationMini + Random(NbIterationRand)) * 1000 div (60 - CodeIntervalMini - CodeIntervalRand div 2);
  FChangingWindowFrequency := ChangingWindowFrequency * 1000 div FInterval;
  FChangingWindowFrequency2 := FChangingWindowFrequency div 4;
  FMatrixState := msOpeningNewWindow;
  FTimer.Interval := FInterval;
  FNbIterations := 0;
  FNbIter := 0;
  EndText := FFallingEndText;
  for I := 1 to MATRIX_WIDTH do
    EndTextInit[I] := False;
  for I := 0 to MATRIX_WIDTH - 1 do
    for J := 0 to MATRIX_HEIGHT - 1 do
    begin
      TheMatrix[I, J].IsLatin := False;
      TheMatrix[I, J].CharNum := Random(NB_GLYPH);
      TheMatrix[I, J].Changing := SPEED_MAX;
      TheMatrix[I, J].HighLighted := 0;
      TheMatrix[I, J].Visible := 0;
      TheMatrix[I, J].BlackTemp := 0;
      TheMatrix[I, J].BlackTempVisible := 0;
    end;
  for I := 0 to FNbRow - 1 do
  begin
    FRows[I].Init;
    FRows[I].StepIt;
  end;
  for I := 0 to NB_SHORT_ROW - 1 do
  begin
    FShortRows[I].Init;
    FShortRows[I].StepIt;
  end;
  FTimer.Interval := FInterval;
  FTimer.OnTimer := StepIt;
  for J := 1 to 50 do
    for I := 0 to FNbRow - 1 do
      FRows[I].StepIt;
end;

//-----------------------------------------------------------------------------

procedure TCodeEmulator.StepIt(Sender: TObject; LagCount: Integer);

//---------------------------------------------------------------------------

  procedure RefreshChars(RandomChange: Boolean = True);
  var
    I, J: Integer;
  begin
    for I := 0 to MATRIX_WIDTH - 1 do
      for J := 0 to MATRIX_HEIGHT - 1 do
      begin
        if (TheMatrix[I, J].Visible > 0) then
          Dec(TheMatrix[I, J].Visible)
        else if (TheMatrix[I, J].Visible < 0) then
          Inc(TheMatrix[I, J].Visible);
        if (TheMatrix[I, J].HighLighted > 0) then
          Dec(TheMatrix[I, J].HighLighted);
        if (TheMatrix[I, J].BlackTemp > 0) then
          Dec(TheMatrix[I, J].BlackTemp);
        if (TheMatrix[I, J].BlackTempVisible > 0) then
          Dec(TheMatrix[I, J].BlackTempVisible);
      end;
    if RandomChange then
      for I := 0 to NB_NEW_GLYPH - 1 do
        with TheMatrix[Random(MATRIX_WIDTH), Random(MATRIX_HEIGHT)] do
        begin
          CharNum := Random(NB_GLYPH);
          Changing := SPEED_MAX;
        end;
  end;

  //---------------------------------------------------------------------------

  procedure DrawAll;
  var
    I, J: Integer;
  begin
    ClearScreen;
    for I := 0 to MATRIX_WIDTH - 1 do
      for J := 0 to MATRIX_HEIGHT - 1 do
      begin
        if TheMatrix[I, J].IsLatin then
          DrawGlyph(TheMatrix[I, J].CharNum mod NB_GLYPH, I * GLYPH_WIDTH, J *
            GLYPH_HEIGHT, 6 + TheMatrix[I, J].CharNum div NB_GLYPH)
        else if (TheMatrix[I, J].HighLighted > SPEED_MAX) then
          DrawGlyph(TheMatrix[I, J].CharNum, I * GLYPH_WIDTH, J *
            GLYPH_HEIGHT, 5)
        else if (TheMatrix[I, J].HighLighted > 0) then
          DrawGlyph(TheMatrix[I, J].CharNum, I * GLYPH_WIDTH, J *
            GLYPH_HEIGHT, 4)
        else if (TheMatrix[I, J].BlackTemp > 0) then
        begin
          if (TheMatrix[I, J].BlackTempVisible > 5) then
            DrawGlyph(TheMatrix[I, J].CharNum, I * GLYPH_WIDTH, J *
              GLYPH_HEIGHT, 1)
          else if (TheMatrix[I, J].BlackTempVisible > 0) then
            DrawGlyph(TheMatrix[I, J].CharNum, I * GLYPH_WIDTH, J *
              GLYPH_HEIGHT, 0)
        end
        else if (abs(TheMatrix[I, J].Visible) > 10) then
          if (TheMatrix[I, J].Changing > 0) then
          begin
            DrawGlyph(TheMatrix[I, J].CharNum, I * GLYPH_WIDTH, J *
              GLYPH_HEIGHT, 3);
            Dec(TheMatrix[I, J].Changing);
          end
          else if (TheMatrix[I, J].Visible > 0) then
            DrawGlyph(TheMatrix[I, J].CharNum, I * GLYPH_WIDTH, J *
              GLYPH_HEIGHT, 2)
          else
            DrawGlyph(TheMatrix[I, J].CharNum, I * GLYPH_WIDTH, J *
              GLYPH_HEIGHT, 4)
        else if (abs(TheMatrix[I, J].Visible) > 5) then
          DrawGlyph(TheMatrix[I, J].CharNum, I * GLYPH_WIDTH, J *
            GLYPH_HEIGHT, 1)
        else if (abs(TheMatrix[I, J].Visible) > 0) then
          DrawGlyph(TheMatrix[I, J].CharNum, I * GLYPH_WIDTH, J *
            GLYPH_HEIGHT, 0)
      end;
    UpdateDisplay;
  end;

  //---------------------------------------------------------------------------

var
  I, J, K, Lag: Integer;
  Ended: Boolean;
  NbTry: Integer;
begin
  //DXMessage := inttostr(FChangingWindowFrequency div FInterval);
  case FMatrixState of
    msFalling:
      begin
        for Lag := 1 to LagCount do
        begin
          RefreshChars;
          Inc(FNbIterations);
          if (FNbIterations > FNbIterationsMax) then
          begin
            {            for I := 0 to MATRIX_WIDTH - 1 do
                          for J := 0 to MATRIX_HEIGHT - 1 do
                            if (TheMatrix[I, J].Visible > 100) then
                              TheMatrix[I, J].Visible := 100;}
            for I := 0 to FNbRow - 1 do
              if (FRows[I].Height > 20) then
                FRows[I].Height := 20;
            FNbIter := 0;
            FErasingEndGlyph := False;
            FMatrixState := msFallEnding;
          end
          else if (Random(FChangingWindowFrequency) = 0) then
          begin
            FMatrixState := msStopped;
            FNbIter := 0;
          end
          else if (Random(FChangingWindowFrequency2) = 0) then
          begin
            FMatrixState := msOpeningNewWindow2;
            FNbIter := 0;
          end;
          for I := 0 to FNbRow - 1 do
            FRows[I].StepIt;
          for I := 0 to NB_SHORT_ROW - 1 do
            FShortRows[I].StepIt(Random(300) = 0);
        end;
        DrawAll;
      end;
    msFallEnding:
      begin
        for Lag := 1 to LagCount do
        begin
          Ended := True;
          for I := 1 to MATRIX_WIDTH do
            Ended := Ended and (EndTextInit[I] or (EndText[I] = '_'));
          if not (Ended) then
          begin
            repeat
              I := Random(MATRIX_WIDTH) + 1;
            until not (EndTextInit[I]) and (EndText[I] <> '_');
            NbTry := 0;
            repeat
              K := Random(FNbRow);
              Inc(NbTry);
            until (NbTry > 200) or (FRows[K].Y >= MATRIX_HEIGHT);
            if (NbTry <= 200) then
              EndTextInit[I] := FRows[K].Init(I - 1, EndText[I]);
          end;
          for I := 0 to FNbRow - 1 do
            FRows[I].StepIt(False);
        end;
        RefreshChars(False);
        DrawAll;
        Ended := True;
        for I := 1 to MATRIX_WIDTH do
          Ended := Ended and (EndTextInit[I] or (EndText[I] = '_'));
        if Ended and not FErasingEndGlyph then
        begin
          FErasingEndGlyph := True;
          for I := 0 to MATRIX_WIDTH - 1 do
            for J := 0 to MATRIX_HEIGHT - 1 do
              if not (TheMatrix[I, J].IsLatin) and (abs(TheMatrix[I, J].Visible) > J * SPEED_MAX) then
                TheMatrix[I, J].Visible := J * SPEED_MAX;
        end;
        K := 0;
        for I := 0 to MATRIX_WIDTH - 1 do
          for J := 0 to MATRIX_HEIGHT - 1 do
            if (abs(TheMatrix[I, J].Visible) > 0) then
              Inc(K);
        if (K < MATRIX_WIDTH) and Ended then
        begin
          for I := 0 to MATRIX_WIDTH - 1 do
            for J := 0 to MATRIX_HEIGHT - 1 do
              if not (TheMatrix[I, J].IsLatin) then
                TheMatrix[I, J].Visible := 0;
          FMatrixState := msFallEnded;
          FNbIter := 0;
          DrawAll;
        end;
      end;
    msFallEnded:
      begin
        for Lag := 1 to LagCount do
          Inc(FNbIter);
        if (FNbIter > 200) then
          Global.NewSequence;
      end;
    msOpeningNewWindow:
      begin
        for Lag := 1 to LagCount do
        begin
          for I := 0 to FNbRow - 1 do
            FRows[I].StepIt;
          for I := 0 to MATRIX_WIDTH - 1 do
            for J := 0 to MATRIX_HEIGHT - 1 do
              if (I in [MATRIX_WIDTH div 2 - NEW_WINDOW_SIDE - MATRIX_WIDTH * FNbIter div (2 * MATRIX_HEIGHT)..MATRIX_WIDTH div 2 + NEW_WINDOW_SIDE - 1 + MATRIX_WIDTH * FNbIter div (2 * MATRIX_HEIGHT)])
              and (J in [MATRIX_HEIGHT div 2 - NEW_WINDOW_SIDE - FNbIter div 2..MATRIX_HEIGHT div 2 + NEW_WINDOW_SIDE - 1 + FNbIter div 2]) then
                if not ((I in [MATRIX_WIDTH div 2 - 1 - MATRIX_WIDTH * FNbIter div (2 * MATRIX_HEIGHT)..MATRIX_WIDTH div 2 + MATRIX_WIDTH * FNbIter div (2 * MATRIX_HEIGHT)])
                  and (J in [MATRIX_HEIGHT div 2 - 1 - FNbIter div 2..MATRIX_HEIGHT div 2 + FNbIter div 2])) then
                  TheMatrix[I, J].HighLighted := SPEED_MAX - 1
                else
              else
              begin
                TheMatrix[I, J].Visible := 0;
                TheMatrix[I, J].HighLighted := 0;
              end;
          RefreshChars;
          Inc(FNbIter);
        end;
        DrawAll;
        if (MATRIX_WIDTH div 2 - NEW_WINDOW_SIDE - MATRIX_WIDTH * FNbIter div (2 * MATRIX_HEIGHT) < 0) or
          (MATRIX_HEIGHT div 2 - NEW_WINDOW_SIDE - FNbIter div 2 < 0) or
          (MATRIX_WIDTH div 2 + NEW_WINDOW_SIDE - 1 + MATRIX_WIDTH * FNbIter div (2 * MATRIX_HEIGHT) >= MATRIX_WIDTH) or
          (MATRIX_HEIGHT div 2 + NEW_WINDOW_SIDE - 1 + FNbIter div 2 >= MATRIX_HEIGHT) then
          FMatrixState := msFalling;
      end;
    msOpeningNewWindow2:
      begin
        for Lag := 1 to LagCount do
        begin
          for I := 0 to FNbRow - 1 do
            FRows[I].StepIt;
          for I := 0 to MATRIX_WIDTH - 1 do
            for J := 0 to MATRIX_HEIGHT - 1 do
              if (I in [MATRIX_WIDTH div 2 - NEW_WINDOW_SIDE - MATRIX_WIDTH * FNbIter div (2 * MATRIX_HEIGHT)..MATRIX_WIDTH div 2 + NEW_WINDOW_SIDE - 1 + MATRIX_WIDTH * FNbIter div (2 * MATRIX_HEIGHT)])
              and (J in [MATRIX_HEIGHT div 2 - NEW_WINDOW_SIDE - FNbIter div 2..MATRIX_HEIGHT div 2 + NEW_WINDOW_SIDE - 1 + FNbIter div 2]) then
                if not ((I in [MATRIX_WIDTH div 2 - 1 - MATRIX_WIDTH * FNbIter div (2 * MATRIX_HEIGHT)..MATRIX_WIDTH div 2 + MATRIX_WIDTH * FNbIter div (2 * MATRIX_HEIGHT)])
                  and (J in [MATRIX_HEIGHT div 2 - 1 - FNbIter div 2..MATRIX_HEIGHT div 2 + FNbIter div 2])) then
                  if (TheMatrix[I, J].HighLighted > 0) or (TheMatrix[I, J].Visible > 0) or (TheMatrix[I, J].BlackTempVisible > 0) then
                    TheMatrix[I, J].HighLighted := SPEED_MAX - 1;
          RefreshChars;
          Inc(FNbIter);
        end;
        DrawAll;
        if (MATRIX_WIDTH div 2 - NEW_WINDOW_SIDE - MATRIX_WIDTH * FNbIter div (2 * MATRIX_HEIGHT) < 0) or
          (MATRIX_HEIGHT div 2 - NEW_WINDOW_SIDE - FNbIter div 2 < 0) then
          FMatrixState := msFalling;
      end;
    msStopped:
      case FNbIter of
        0:
          begin
            for I := 0 to MATRIX_WIDTH - 1 do
              for J := 0 to MATRIX_HEIGHT - 1 do
              begin
                if (abs(TheMatrix[I, J].Visible) > 0) then
                  TheMatrix[I, J].Visible := 6;
                if (TheMatrix[I, J].HighLighted > 0) then
                  TheMatrix[I, J].HighLighted := SPEED_MAX - 1;
                TheMatrix[I, J].BlackTemp := 0;
              end;
            Inc(FNbIter);
          end;
        1..7:
          begin
            for I := 0 to MATRIX_WIDTH - 1 do
              for J := 0 to MATRIX_HEIGHT - 1 do
                if (TheMatrix[I, J].Visible > 0) then
                  Dec(TheMatrix[I, J].Visible)
                else if (TheMatrix[I, J].Visible < 0) then
                  Inc(TheMatrix[I, J].Visible);
            Inc(FNbIter);
            DrawAll;
          end;
        8..50: Inc(FNbIter);
        else
          begin
            FNbRow := NbRowMini + Random(NbRowRand);
            FInterval := 60 - (CodeIntervalMini + Random(CodeIntervalRand));
            FTimer.Interval := FInterval;
            FChangingWindowFrequency := ChangingWindowFrequency * 1000 div FInterval;
            FChangingWindowFrequency2 := FChangingWindowFrequency div 4;
            FMatrixState := msOpeningNewWindow;
            FNbIter := 0;
          end;
      end;
    msDejaVu:
      begin
      end;
  end;
end;

//-----------------------------------------------------------------------------

procedure TCodeEmulator.OpenWindow;
begin
  FMatrixState := msStopped;
  FNbIter := 0;
end;

//-----------------------------------------------------------------------------

end.

