unit Global;

interface

const
  ALPHA_WIDTH = 20;
  ALPHA_HEIGHT = 20;
  GLYPH_WIDTH = 12;
  GLYPH_HEIGHT = 16;
  NEW_WINDOW_SIDE = 4;
  END_OF_LINE_INTERVAL = 1000;
  END_TEXT_INTERVAL = 2500;
  PHONE_SEQ_INTERVAL = 10;
  CURSOR_INTERVAL = 100;

  MATRIX_WIDTH_MAX = 133;
  MATRIX_HEIGHT_MAX = 100;
  NB_SHORT_ROW_MAX = 200;
  NB_ROW_MAX_MAX = 392;

//  MATRIX_WIDTH = 85;
//  MATRIX_HEIGHT = 48;
//  NB_NEW_GLYPH = 50;
  NB_GLYPH = 48;
//  NB_ROW_MAX = 250;
//  NB_SHORT_ROW = 50;
//  SPEED_MAX = 6;
  RESOL_DEF = 1;
  CHARACTER_DENSITY_DEF = 24;
  CHARACTER_DENSITY_DELTA_DEF = 2;
  FALLING_SPEED_DEF = 50;
  FALLING_SPEED_DELTA_DEF = 2;
  WRITING_SPEED_DEF = 70;
  FALLING_CODE_DURATION_DEF = 180;
  FALLING_CODE_DURATION_DELTA_DEF = 2;
  CHANGING_WINDOW_FREQUENCY_DEF = 30;
  FALLING_TEXT_DEF = '____________________M_A_T_R_I_X___________________';
  SHOW_FPS_DEF = False;
  PHONE_NUMBER_DEF = '5558462871';
  FINDING_NUMBER_SPEED_DEF = 50;
  PHONE_SEQUENCE_ENABLED_DEF = True;
  TRINITY_TEXT_ENABLED_DEF = True;
  TRINITY_TEXT_DEF = 'Wake up Neo...' + #13#10 + 'The matrix has you...'
    + #13#10 + 'Follow the white rabbit.' + #13#10 + 'Knock, knock, Neo.';

var
  CfgFile: string;
  NbRowMini: Integer;
  NbRowRand: Integer;
  CodeIntervalMini: Integer;
  CodeIntervalRand: Integer;
  NbIterationMini: Integer;
  NbIterationRand: Integer;
  ChangingWindowFrequency: Integer;
  FallingEndText: string;
  DXMessage: string;
  MATRIX_WIDTH: Integer;
  MATRIX_HEIGHT: Integer;
  NB_NEW_GLYPH: Integer;
  NB_ROW_MAX: Integer;
  NB_SHORT_ROW: Integer;
  SPEED_MAX: Integer;
  PHONE_WIDTH: Integer;
  PHONE_HEIGHT: Integer;

procedure NewSequence;
procedure ClearScreen;
procedure UpdateDisplay;
procedure DrawGlyph(CharNum: Integer; X, Y: Integer; Style: Integer);
procedure DrawAlpha(Ch: Char; X, Y: Integer);
procedure DrawSysFail(X, Y: Integer);
procedure DrawNum(Ch: Char; X, Y: Integer; HighLight: Boolean);

implementation

uses Graphics, Main, SysUtils, Types;

//-----------------------------------------------------------------------------

procedure NewSequence;
begin
  with MainForm do
    if ((ScreenState = ssCodeEmulator) and (PhoneSequenceEnabled or TrinityTextEnabled)) then
      if (((Random(2) = 0) and PhoneSequenceEnabled) or not (TrinityTextEnabled)) then
      begin
        PhoneSequence.NewSequence;
        ScreenState := ssPhoneSequence;
      end
      else
      begin
        TrinityText.NewSequence;
        ScreenState := ssTrinitysTalking;
      end
    else
    begin
      CodeEmulator.NewSequence;
      ScreenState := ssCodeEmulator;
    end;
end;

//-----------------------------------------------------------------------------

procedure ClearScreen;
begin
  MainForm.DXDraw.Surface.Fill(0);
end;

//-----------------------------------------------------------------------------

procedure UpdateDisplay;
begin
  if (MainForm.ShowFPS) or (DXMessage <> '') then
    with MainForm.DXDraw.Surface.Canvas do
    begin
      Brush.Style := bsSolid;
      Brush.Color := clBlack;
      Font.Color := clWhite;
      Font.Size := 12;
      if MainForm.ShowFPS then
      begin
        Textout(0, 0, 'FPS: ' + inttostr(MainForm.Timer.FrameRate));
        if DXMessage <> '' then
          Textout(0, 20, DXMessage);
      end
      else
        Textout(0, 0, DXMessage);
      Release; //  Indispensability
    end;
  MainForm.DXDraw.Flip;
end;

//-----------------------------------------------------------------------------

procedure DrawGlyph(CharNum: Integer; X, Y: Integer; Style: Integer);
begin
  MainForm.DXDraw.Surface.Draw(X, Y,
    Rect(CharNum * GLYPH_WIDTH, Style * GLYPH_HEIGHT, (CharNum + 1) * GLYPH_WIDTH, (Style + 1) * GLYPH_HEIGHT),
    MainForm.GlyphSurface, True);
  if Style = 5 then
    MainForm.DXDraw.Surface.DrawAlpha(Rect(X - 6, Y - 4, X + 17, Y + 19), Rect(0, 0, 23, 23), MainForm.WhiteCircleSurface, True, 128);
end;

//-----------------------------------------------------------------------------

procedure DrawAlpha(Ch: Char; X, Y: Integer);
var
  MyRect: TRect;
begin
  MyRect := Rect((Ord(Ch) - Ord(' ')) * ALPHA_WIDTH, 0, (Ord(Ch) - Ord(' ') + 1) * ALPHA_WIDTH, ALPHA_HEIGHT);
  MainForm.DXDraw.Surface.Draw(X, Y, MyRect, MainForm.AlphaSurface, False);
end;

//-----------------------------------------------------------------------------

procedure DrawSysFail(X, Y: Integer);
begin
  MainForm.DXDraw.Surface.Draw(X, Y, Rect(0, 0, 239, 39), MainForm.SysFailSurface, False);
end;

//-----------------------------------------------------------------------------

procedure DrawNum(Ch: Char; X, Y: Integer; HighLight: Boolean);
var
  MyRect: TRect;
begin
  if (HighLight) then
    MyRect := Rect((Ord(Ch) - Ord('0')) * ALPHA_WIDTH, ALPHA_HEIGHT + 1, (Ord(Ch) - Ord('0') + 1) * ALPHA_WIDTH - 1, 2 * ALPHA_HEIGHT - 1)
  else
    MyRect := Rect((Ord(Ch) - Ord('0')) * ALPHA_WIDTH, 0, (Ord(Ch) - Ord('0') + 1) * ALPHA_WIDTH - 1, ALPHA_HEIGHT - 1);
  MainForm.DXDraw.Surface.Draw(X, Y, MyRect, MainForm.NumSurface, False);
end;

//-----------------------------------------------------------------------------

begin
  CfgFile := ExtractFilePath(ParamStr(0)) + 'matrix.ini';
  DXMessage := '';
end.

