unit Phone;

interface

uses Global, Classes, DIB, DXDraws, DXClass, Windows, SysUtils;

type
  TPhoneSequence = class
  private
    FPhoneNumberToFind: string[10];
    FPhoneNumber: string[10];
    FLines: TStringList;
    FLetterNum: Word;
    FLineNum: Word;
    FTimer: TDXTimer;
    FPhoneSeqEnded: Boolean;
    FDisplayingText: Boolean;
    FCursorBlinking: Boolean;
    FNbCursorBlink: Integer;
    FFindingNumberSpeed: Integer;
    FSysFailProbability: Integer;
  public
    constructor Create(PhoneNumber: string; FindingNumberSpeed: Integer;
      Timer: TDXTimer);
    destructor Destroy; override;
    procedure NewSequence;
    procedure StepIt(Sender: TObject; LagCount: Integer);
    procedure EndIt;
  end;

implementation

//-----------------------------------------------------------------------------

constructor TPhoneSequence.Create(PhoneNumber: string; FindingNumberSpeed: Integer; Timer: TDXTimer);
begin
  FFindingNumberSpeed := 110 - FindingNumberSpeed;
  FSysFailProbability := 10 * FFindingNumberSpeed;
  FLines := TStringList.Create;
  FPhoneNumberToFind := PhoneNumber;
  FTimer := Timer;
  DateSeparator := '-';
  FLines.Clear;
  FLines.Add('Call trans opt:received.' + DateToStr(Date) + ' ' + TimeToStr(Time) + ' RBC:Log>');
  FLines.Add('WARNING: carrier anomaly');
  FLines.Add('Trace program: running');
end;

//-----------------------------------------------------------------------------

destructor TPhoneSequence.Destroy;
begin
  FLines.Free;
end;

//-----------------------------------------------------------------------------

procedure TPhoneSequence.NewSequence;
begin
  FLines[0] := 'Call trans opt:received.' + DateToStr(Date) + ' ' + Copy(TimeToStr(Time), 1, 5) + ' RBC:Log>';
  FPhoneNumber := '          ';
  FLetterNum := 1;
  FLineNum := 0;
  FDisplayingText := True;
  FPhoneSeqEnded := False;
  ClearScreen;
  FTimer.Interval := PHONE_SEQ_INTERVAL;
  FTimer.OnTimer := StepIt;
end;

//-----------------------------------------------------------------------------

procedure TPhoneSequence.StepIt(Sender: TObject; LagCount: Integer);
var
  I, J, K: Integer;
  Ch: Char;
  NumberFound: Boolean;
  NbNumberFound: Integer;
begin
  if FDisplayingText then
    if FCursorBlinking then
    begin
      Dec(FNbCursorBlink);
      if FLineNum > 2 then
        case (FNbCursorBlink mod 2) of
          0: DrawAlpha(' ', (FLetterNum - 1) * ALPHA_WIDTH, 1 + 20 * FLineNum);
          1: DrawAlpha(Chr(127), (FLetterNum - 1) * ALPHA_WIDTH, 1 + 20 * FLineNum);
        end
      else
        case (FNbCursorBlink mod 2) of
          0: DrawAlpha(' ', (FLetterNum - 1) * ALPHA_WIDTH, 3 + 20 * FLineNum);
          1: DrawAlpha(Chr(127), (FLetterNum - 1) * ALPHA_WIDTH, 3 + 20 * FLineNum);
        end;
      if (FNbCursorBlink < 1) then
        if FPhoneSeqEnded then
          Global.NewSequence
        else
        begin
          FCursorBlinking := False;
          FTimer.Interval := PHONE_SEQ_INTERVAL;
        end;
    end
    else
    begin
      if ((FLines.Count > 0) and (FLetterNum > Length(FLines[FLineNum]))) then
      begin
        Inc(FLineNum);
        FLetterNum := 1;
      end;
      if (FLineNum > FLines.Count - 1) then
        FDisplayingText := False
      else
      begin
        Ch := FLines[FLineNum][FLetterNum];
        if (Ch in [' '..'~']) then
        begin
          DrawAlpha(Ch, (FLetterNum - 1) * ALPHA_WIDTH, 3 + 20 * FLineNum);
          Inc(FLetterNum);
          DrawAlpha(Chr(127), (FLetterNum - 1) * ALPHA_WIDTH, 3 + 20 * FLineNum);
        end;
        if (FLetterNum > Length(FLines[FLineNum])) then
        begin
          FCursorBlinking := True;
          FNbCursorBlink := 10;
          FTimer.Interval := CURSOR_INTERVAL;
        end;
      end;
    end
  else
  begin
    NbNumberFound := 0;
    for I := 1 to 10 do
      if (FPhoneNumber[I] <> ' ') then Inc(NbNumberFound);
    NumberFound := (NbNumberFound = 10);
    if NumberFound then
    begin
      FCursorBlinking := True;
      FDisplayingText := True;
      FLineNum := 4;
      FLetterNum := 11;
      FNbCursorBlink := 50;
      FTimer.Interval := CURSOR_INTERVAL;
      FPhoneSeqEnded := True;
    end;
    for I := 0 to PHONE_WIDTH do
      for J := 4 to PHONE_HEIGHT do
        if ((I < 10) and (J = 4)) then
          DrawNum(FPhoneNumber[I + 1], I * ALPHA_WIDTH, 3 + 20 * J, False)
        else if (FPhoneNumber[I mod 10 + 1] = ' ') then
          DrawNum(Chr(Random(10) + Ord('0')), I * ALPHA_WIDTH, 3 + 20 * J, Random(7) = 0)
        else
          DrawAlpha(' ', I * ALPHA_WIDTH, 3 + 20 * J);
    if ((Random(FFindingNumberSpeed) = 0) and not (NumberFound)) then
    begin
      repeat K := Random(10);
      until (FPhoneNumber[K + 1] = ' ');
      FPhoneNumber[K + 1] := FPhoneNumberToFind[K + 1];
    end;
    if ((Random(FSysFailProbability) = 0) and (NbNumberFound in [3..6])) then
    begin
      DrawSysFail(400, 400);
      FCursorBlinking := True;
      FDisplayingText := True;
      FLineNum := 0;
      FLetterNum := 0;
      FNbCursorBlink := 0;
      FTimer.Interval := 50 * CURSOR_INTERVAL;
      FPhoneSeqEnded := True;
    end;
  end;
  UpdateDisplay;
end;

//-----------------------------------------------------------------------------

procedure TPhoneSequence.EndIt;
begin
  Global.NewSequence
end;

//-----------------------------------------------------------------------------

end.

